package com.gic.haoban.manage.service.dao.mapper.role;

import com.gic.haoban.manage.service.entity.role.TabHaobanMenu;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author mozhu
 * @since 2022-09-06 16:35:10
 */
public interface TabHaobanMenuMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param menuCode 主键
     * @return 实例对象
     */
    TabHaobanMenu queryById(String menuCode);

    /**
     * 新增数据
     *
     * @param tabHaobanMenu 实例对象
     * @return 影响行数
     */
    int insert(TabHaobanMenu tabHaobanMenu);

    /**
     * 批量新增数据（MyBatis原生foreach方法）
     *
     * @param entities List<TabHaobanMenu> 实例对象列表
     * @return 影响行数
     */
    int insertBatch(@Param("entities") List<TabHaobanMenu> entities);

    /**
     * 修改数据
     *
     * @param tabHaobanMenu 实例对象
     * @return 影响行数
     */
    int update(TabHaobanMenu tabHaobanMenu);

    List<TabHaobanMenu> getAllMenuList();

    List<TabHaobanMenu> getPidMenuList();

    List<TabHaobanMenu> getNotPidMenuList();


    /**
     * 得到haoban菜单列表
     *
     * @param wxEnterpriseId wx企业标识
     * @param clerkType      职员类型
     * @return {@link List}<{@link TabHaobanMenu}>
     */
    List<TabHaobanMenu> getHaobanPidMenuList(@Param("wxEnterpriseId") String wxEnterpriseId,
                                             @Param("clerkType") Integer clerkType,
                                             @Param("systemFlag") Integer systemFlag);

    List<TabHaobanMenu> getHaobanNotPidMenuList(@Param("wxEnterpriseId") String wxEnterpriseId,
                                                @Param("clerkType") Integer clerkType,
                                                @Param("systemFlag") Integer systemFlag);

    /**
     * 得到haoban菜单列表
     *
     * @param wxEnterpriseId wx企业标识
     * @param clerkType      职员类型
     * @return {@link List}<{@link TabHaobanMenu}>
     */
    List<String> getHaobanMenuListByClerkType(@Param("wxEnterpriseId") String wxEnterpriseId,
                                              @Param("clerkType") Integer clerkType,
                                              @Param("systemFlag") Integer systemFlag);

    /**
     * 获取菜单code
     *
     * @param menuCodes 菜单代码
     * @return {@link List}<{@link String}>
     */
    List<String> getHaobanMenuCodeListByCode(@Param("menuCodes") List<String> menuCodes);

    /**
     * 获取子菜单code
     *
     * @param menuCodes 菜单代码
     * @return {@link List}<{@link String}>
     */
    List<String> getHaobanChildMenuCodeByPCode(@Param("menuCodes") List<String> menuCodes);

    /**
     * 获取菜单列表
     *
     * @param menuCodes 菜单代码
     * @return {@link List}<{@link String}>
     */
    List<TabHaobanMenu> getHaobanMenuListByCode(@Param("menuCodes") List<String> menuCodes);

    TabHaobanMenu getMenuByCode(@Param("menuCode") String menuCode);

    void deleteAll();

    void deleteByMenuCode(@Param("menuCode")String menuCode) ;
}

