package com.gic.haoban.manage.web.handle;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.DataApiUtils;
import com.gic.enterprise.api.dto.security.DownloadReportDTO;
import com.gic.haoban.manage.web.vo.chat.GroupChatClerkVO;
import com.gic.haoban.manage.web.vo.chat.GroupChatDetailVO;
import com.gic.haoban.manage.web.vo.chat.GroupChatJoinDetailVO;
import com.gic.web.common.download.DownloadHandlerAbstract;
import com.gic.web.common.download.context.Context;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @author shangfeng
 * @date 2024-06-21 11:34:49
 */
@Component
public class GroupJoinHandle extends DownloadHandlerAbstract<GroupChatJoinDetailVO> {

    private static final Logger LOGGER = LogManager.getLogger(GroupJoinHandle.class);


    @Override
    public List<GroupChatJoinDetailVO> getData(Context context, String searchDataParams, Integer currentPage) {
        JSONObject jsonObject = JSON.parseObject(searchDataParams);
        jsonObject.put("pageNum", currentPage);
        jsonObject.put("pageSize", getPageSize());
        Map<String, Object> res = DataApiUtils.http(jsonObject.toJSONString(), "data_mbr_scale_haoban_qw_group_chat_join");
        List<JSONObject> list = DataApiUtils.getPageList(res);

        if (list == null) {
            list = new ArrayList<>();
        }

        if (CollectionUtils.isNotEmpty(list)) {
            List<GroupChatJoinDetailVO> result = JSONObject.parseArray(JSON.toJSONString(list), GroupChatJoinDetailVO.class);
            result.forEach(groupChatJoinDetailVO -> {
                if (StringUtils.isNotEmpty(groupChatJoinDetailVO.getUserType())) {
                    groupChatJoinDetailVO.setUserType(getUserTypeDesc(groupChatJoinDetailVO.getUserType()));
                }
                if (StringUtils.isNotEmpty(groupChatJoinDetailVO.getJoinScene())) {
                    groupChatJoinDetailVO.setJoinScene(getJoinSceneDesc(groupChatJoinDetailVO.getJoinScene()));
                }
            });
            return result;
        }
        return Lists.newArrayList();
    }

    @Override
    public List<String> getColumnNames() {
        return Lists.newArrayList();
    }

    @Override
    public List<String> getColumnNames(Context context) {
        DownloadReportDTO downloadReportDTO = context.getDownloadReportDTO();
        String searchDataParams = downloadReportDTO.getSearchDataParams();
        JSONObject jsonObject = JSONObject.parseObject(searchDataParams);
        List<String> list = (List<String>) jsonObject.get("indexList");
        return list;
    }

    @Override
    public LinkedHashMap<String, List<String>> doubleColumnNames() {
        return null;
    }

    @Override
    public List<String> getColumns() {
        return Lists.newArrayList();
    }

    @Override
    public List<String> getColumns(Context context) {
        DownloadReportDTO downloadReportDTO = context.getDownloadReportDTO();
        String searchDataParams = downloadReportDTO.getSearchDataParams();
        JSONObject jsonObject = JSONObject.parseObject(searchDataParams);
        List<String> list = (List<String>) jsonObject.get("fieldList");
        return list;
    }

    private String getUserTypeDesc(String userType) {
        switch (userType) {
            case "1":
                return "内部成员";
            case "2":
                return "外部成员";
            case "3":
                return "客户";
            default:
                return "";
        }
    }


    private String getJoinSceneDesc(String userType) {
        switch (userType) {
            case "1":
                return "群成员直接邀请入群";
            case "2":
                return "群成员邀请链接入群";
            case "3":
                return "通过扫描群二维码入群";
            default:
                return "";
        }
    }

}
