package com.gic.haoban.manage.web.handle;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.DataApiUtils;
import com.gic.enterprise.api.dto.security.DownloadReportDTO;
import com.gic.haoban.manage.web.vo.chat.SingleChatStoreGroupVO;
import com.gic.haoban.manage.web.vo.wechatwork.WechatWorkLostClerkVO;
import com.gic.web.common.download.DownloadHandlerAbstract;
import com.gic.web.common.download.context.Context;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author shangfeng
 * @date 2024-06-21 11:33:18
 */
@Component
public class SingleChatStoreDataHandle extends DownloadHandlerAbstract<SingleChatStoreGroupVO> {

    private static final Logger LOGGER = LogManager.getLogger(SingleChatStoreDataHandle.class);


    @Override
    public List<SingleChatStoreGroupVO> getData(Context context, String searchDataParams, Integer currentPage) {
        boolean isFirst = currentPage != null && currentPage == 1;
        if (isFirst) {
            LOGGER.info("好办单聊统计-门店维度-导出日志");
        }
        JSONObject jsonObject = JSON.parseObject(searchDataParams);
        jsonObject.put("pageNum", currentPage);
        jsonObject.put("pageSize", getPageSize());
        Map<String, Object> res = DataApiUtils.http(jsonObject.toJSONString(), "data_mbr_scale_haoban_qw_chat_count");
        List<JSONObject> list = DataApiUtils.getPageList(res);

        jsonObject.put("groupType", 1);
        jsonObject.put("storeGroup", 8);

        if (list == null) {
            list = new ArrayList<>();
        }
        if (isFirst) {
            Map<String, Object> totalRes = DataApiUtils.http(jsonObject.toJSONString(), "data_mbr_scale_haoban_qw_chat_count");
            Page page = DataApiUtils.getPageData(totalRes);
            if (page != null) {
                List<JSONObject> totalList = page.getResult();
                if (CollectionUtils.isNotEmpty(totalList)) {
                    JSONObject total = totalList.get(0);
                    total.put("clerkName", "合计（" + page.getTotalCount() + "）");
                    list.add(0, total);
                }
            }
        }

        if (CollectionUtils.isNotEmpty(list)) {
            List<SingleChatStoreGroupVO> result = JSONObject.parseArray(JSON.toJSONString(list), SingleChatStoreGroupVO.class);
            return result;
        }
        return Lists.newArrayList();
    }

    @Override
    public List<String> getColumnNames() {
        return Lists.newArrayList();
    }

    @Override
    public List<String> getColumnNames(Context context) {
        DownloadReportDTO downloadReportDTO = context.getDownloadReportDTO();
        String searchDataParams = downloadReportDTO.getSearchDataParams();
        JSONObject jsonObject = JSONObject.parseObject(searchDataParams);
        List<String> list = (List<String>) jsonObject.get("indexList");
        return list;
    }

    @Override
    public LinkedHashMap<String, List<String>> doubleColumnNames() {
        return null;
    }

    @Override
    public List<String> getColumns() {
        return Lists.newArrayList();
    }

    @Override
    public List<String> getColumns(Context context) {
        DownloadReportDTO downloadReportDTO = context.getDownloadReportDTO();
        String searchDataParams = downloadReportDTO.getSearchDataParams();
        JSONObject jsonObject = JSONObject.parseObject(searchDataParams);
        List<String> list = (List<String>) jsonObject.get("fieldList");
        return list;
    }
}
