package com.gic.haoban.manage.web.utils;


import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Excel操作
 *
 * @author 徐波  2019-4-18
 */
public class ImportExcelUtil {

    private static final Logger log = LoggerFactory.getLogger(ImportExcelUtil.class);

    private final static String excel2003L = ".xls";    //2003- 版本的excel
    private final static String excel2007U = ".xlsx";   //2007+ 版本的excel

    /**
     * 描述：获取IO流中的数据，组装成List<List<Object>>对象
     *
     * @return 获取第一个sheet 中数据
     */
    public static void getBankListByExcelSheet1(InputStream in, String fileName, List<List<Object>> sheet1List) throws Exception {

        //创建Excel工作薄
        Workbook work = getWorkbook(in, fileName);
        if (null == work) {
            throw new Exception("创建Excel工作薄为空！");
        }
        Sheet sheet1 = null;
        Row row1 = null;
        Cell cell1 = null;
        //遍历Excel中所有的sheet
        sheet1 = work.getSheetAt(0);
        if (sheet1 != null) {
            //遍历当前sheet中的所有行
            for (int j = sheet1.getFirstRowNum(); j <= sheet1.getLastRowNum(); j++) {
                row1 = sheet1.getRow(j);
                if (row1 == null) {
                    continue;
                }
                //遍历所有的列
                List<Object> li = new ArrayList<Object>();
                log.info("单元格数={}", row1.getFirstCellNum());
                if(row1.getFirstCellNum() != 0) {
                    for(int i=0;i<(row1.getFirstCellNum())  ; i++) {
                        li.add("") ;
                    }
                }
                for (int y = row1.getFirstCellNum(); y < row1.getLastCellNum(); y++) {
                    cell1 = row1.getCell(y);
                    //1、代表单元格里面类型为String类型
                    if (cell1 != null) {
                        li.add(getCellValue(cell1));
                    } else {
                        li.add("");
                    }
                }
                if (CollectionUtils.isNotEmpty(li)) {
                    sheet1List.add(li);
                }
            }
        }
    }

    /**
     * 描述：根据文件后缀，自适应上传文件的版本
     *
     * @param inStr,fileName
     * @return
     * @throws Exception
     */
    public static Workbook getWorkbook(InputStream inStr, String fileName) throws Exception {
        Workbook wb = null;
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        if (excel2003L.equals(fileType)) {
            wb = new HSSFWorkbook(inStr);  //2003-
        } else if (excel2007U.equals(fileType)) {
            wb = new XSSFWorkbook(inStr);  //2007+
        } else {
            throw new Exception("解析的文件格式有误！");
        }
        return wb;
    }

    /**
     * 描述：对表格中数值进行格式化
     *
     * @param cell
     * @return
     */
    public static Object getCellValue(Cell cell) {
        Object value = null;
        DecimalFormat df = new DecimalFormat("0");  //格式化number String字符
        SimpleDateFormat sdf = new SimpleDateFormat("yyy-MM-dd");  //日期格式化
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyy-MM-dd HH:mm");  //日期格式化
        DecimalFormat df2 = new DecimalFormat("0.00");  //格式化数字

        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_STRING:
                value = cell.getRichStringCellValue().getString();
                break;
            case Cell.CELL_TYPE_NUMERIC:
                if ("General".equals(cell.getCellStyle().getDataFormatString())) {
                    value = df.format(cell.getNumericCellValue());
                } else if ("m/d/yy".equals(cell.getCellStyle().getDataFormatString())) {
                    value = sdf.format(cell.getDateCellValue());
                } else if (StringUtils.equals("m/d/yy h:mm", cell.getCellStyle().getDataFormatString())) {
                    value = dateFormat.format(cell.getDateCellValue());
                } else {
                    value = df2.format(cell.getNumericCellValue());
                }
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                value = cell.getBooleanCellValue();
                break;
            case Cell.CELL_TYPE_BLANK:
                value = "";
                break;
            default:
                break;
        }
        return value;
    }
}